/*
Roman Yu. Toropov
scheme - ac:key_adc

*/
// LCD module connections
#include <built_in.h>
#include "ee_float.h"

sbit LCD_RS at PORTC4_bit;
sbit LCD_EN at PORTC5_bit;
sbit LCD_D4 at PORTC0_bit;
sbit LCD_D5 at PORTC1_bit;
sbit LCD_D6 at PORTC2_bit;
sbit LCD_D7 at PORTC3_bit;

sbit LCD_RS_Direction at DDC4_bit;
sbit LCD_EN_Direction at DDC5_bit;
sbit LCD_D4_Direction at DDC0_bit;
sbit LCD_D5_Direction at DDC1_bit;
sbit LCD_D6_Direction at DDC2_bit;
sbit LCD_D7_Direction at DDC3_bit;
// End LCD module connections

unsigned int adc_rd, adc_rr, adc_r, keys;


void keyb() {
       Delay_ms(50);
       ADMUX = 64; // 64 = Port F0,  ADMUX +1=F1, +2 = F2,  . .
       ADCSRA = 128+64+7;
       adc_rd = ADCL;
       adc_rr = ADCH;
       adc_r = adc_rd + 256*adc_rr;
       keys = 0;
       if ((adc_r > 450)&&(adc_r < 650)) { keys = 1; }   //"Select"
       if ((adc_r > 300)&&(adc_r < 450)) { keys = 2; }   //"left"
       if ((adc_r > 200)&&(adc_r < 300)) { keys = 3; }   //"Down"
       if ((adc_r > 50)&&(adc_r < 150)) { keys = 4; }    // "Up"
       if (adc_r < 50) { keys = 5; }      //"Right"
       Delay_ms(50);
}

// ADC module connections
sbit Chip_Select_Direction at DDB0_bit;
sbit Chip_Select at PORTB0_bit;

float measurement1, measurement2, measurement3, measurement4, tmpp, niz, verh, tok;
char tokk[7], measuremen1[15], measuremen2[15], measuremen3[15], measuremen4[15];
unsigned int tmpp1, tmpp2, tmpp3, tok1, tok2, tok3, tok4, tok5, next;
bit cyc;

void init_adc() {

  measurement1 = 0;
  measurement2 = 0;
  measurement3 = 0;
  measurement4 = 0;                               // Initialize the measurement variable

// Init SPI
  SPI1_Init_Advanced(_SPI_MASTER,                // Initialize AVR as master
                     _SPI_FCY_DIV16,             // Data sample at end
                     _SPI_CLK_LO_LEADING);

  Chip_Select_Direction = 1;                     // SetBit(DDRB,0)
  Chip_Select= 1;                                // SetBit(PORTB,0)

}

float getADC(unsigned short channel) {    // Returns 0..4095
  unsigned int tmp;
  Chip_Select= 0;                                // Select MCP3204
  SPI1_Write(0x06);                              // SPI communication using 8-bit segments
  channel = channel << 6;                        // Bits 7 & 6 define ADC input
  tmp = SPI1_Read(channel) & 0x0F;
  tmp = tmp << 8;                                // Get ADC value
  tmp |= SPI1_Read(0);
  Chip_Select= 1;
  Delay_us(120);
  return tmp;
}

void adc_read() {

     tmpp1 = getADC(0);tmpp2 = getADC(0);tmpp3 = getADC(0);
     tmpp = (tmpp1+tmpp2+tmpp3)/3;
     measurement1 = tmpp/200;
     sprintf(measuremen1,"%2.2f", measurement1);

     tmpp1 = getADC(1);tmpp2 = getADC(1);tmpp3 = getADC(1);
     tmpp = (tmpp1+tmpp2+tmpp3)/3;
     measurement2 = tmpp/1000;
     sprintf(measuremen2,"%2.3f", measurement2);

     tok1 = getADC(2);
     tok2 = getADC(2);
     tok3 = getADC(2);
     tok = (tok1+tok2+tok3)/3;
     if (tok < niz) {measurement3 = (niz - tok) /100; sprintf(measuremen3,"%2.2f", measurement3); }
     if (tok > verh) {measurement3 = (tok - verh) /100; sprintf(measuremen3,"%2.2f", measurement3); }
     if (tok > niz && tok < verh) {measurement3 = 0.00; sprintf(measuremen3,"%2.2f", measurement3);}

     tmpp1 = getADC(3);tmpp2 = getADC(3);tmpp3 = getADC(3);
     tmpp = (tmpp1+tmpp2+tmpp3)/3;
     measurement4 = tmpp/1000;
     sprintf(measuremen4,"%2.2f", measurement4);


}

void calibrate() {
  Lcd_Cmd(_LCD_CLEAR);
  Delay_ms(500);
  cyc = 1;
  next = 256;
  E2P_write_int(0x00,2400);
  E2P_write_int(0x02,2600);

  while(cyc) {
  adc_read();
  if (tok1 > 2600) { cyc = 0; break ;}
  if (tok1 < 2400) { cyc = 0; break ;}

  tok4=E2P_read_int(0x00);
  if (tok1 > tok4) {E2P_write_int(0x00,tok1+5);}
  tok4=E2P_read_int(0x00);
  if (tok2 > tok4) {E2P_write_int(0x00,tok2+5);}
  tok4=E2P_read_int(0x00);
  if (tok3 > tok4) {E2P_write_int(0x00,tok3+5);}
  tok4=E2P_read_int(0x00);
  if (tok4 < 2400) {E2P_write_int(0x00,2500);}
  tok4=E2P_read_int(0x00);
  IntToStr(tok4, tokk);
  Lcd_Out(1,2,tokk);

  tok5=E2P_read_int(0x02);
  if (tok1 < tok5) {E2P_write_int(0x02,tok1-5);}
  tok5=E2P_read_int(0x02);
  if (tok2 < tok5) {E2P_write_int(0x02,tok2-5);}
  tok5=E2P_read_int(0x02);
  if (tok3 < tok5) {E2P_write_int(0x02,tok3-5);}
  tok5=E2P_read_int(0x02);

  if (tok5 < 2400) {E2P_write_int(0x02,2500);}
  tok5=E2P_read_int(0x02);

  // sprintf(tokk,"%2u", tok5);
  IntToStr(tok5, tokk);
  Lcd_Out(1,8,tokk);
  // Read_t1();
  next--;
  // sprintf(tokk,"%2u", next);
  IntToStr(next, tokk);
  Lcd_Out(2,8,tokk);
  IntToStr(tok1, tokk);
  Lcd_Out(2,2,tokk);
  if (next < 1) {cyc = 0;}
  }
  Lcd_Cmd(_LCD_CLEAR);
  verh=E2P_read_int(0x00);
  niz=E2P_read_int(0x02);
  keyb();
  Delay_ms(500);
}


void main() {
       Lcd_Init();                        // Initialize LCD
       Lcd_Cmd(_LCD_CLEAR);               // Clear display
       Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
       ADCSRA = 128+7;
       init_adc();
       keys=0;
       calibrate();
       verh=E2P_read_int(0x00);
       niz=E2P_read_int(0x02);

     while(1){
     keyb();
     adc_read();
     if (keys==1) {Lcd_Out(1,1,"Select");}
     if (keys==2) {Lcd_Out(1,1,"left  ");}
     if (keys==3) {Lcd_Out(1,1,"Down  ");}
     if (keys==4) {Lcd_Out(1,1,"Up    ");}
     if (keys==5) {Lcd_Out(1,1,"Right ");}
     Lcd_Out(2,9,measuremen1);
     if (measurement1<10) {Lcd_Out(1,13," ");}
     Lcd_Out(2,14,"V");
     Lcd_Out(2,3,measuremen3);
     if (measurement3<10) {Lcd_Out(2,6," ");}
     Lcd_Out(2,7,"A ");

     }


}